import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;

public class ConfigForm extends JDialog implements ActionListener
{
	final private int MAC_ADDR_LEN = 6;
	
	private JPanel jpAdd;
	private JPanel jpRmv;
	private JPanel jpConfigPanel;
	
	private JLabel jlAddMac;
	private JLabel jlRmvOwner;

	private JButton jbOk;
	private JButton jbCancel;
	private JButton jbApply;
	
	private JTextField jtfAddMac[];
	private JTextField jtfRmvOwner;

	private GridBagLayout gblLayout;
	private GridBagConstraints gbcConst;

	private PriorityDriverParameters pdpValues;

	private PriorityStatus psStatus;
	
	private PriorityStatusForm psfForm;
	
	ConfigForm(JFrame jfOwner, PriorityDriverParameters pdpValues, PriorityStatus psStatus, PriorityStatusForm psfForm)
	{
		super(jfOwner, "Configure", true);
		this.pdpValues = pdpValues; 
		this.psStatus = psStatus;
		this.psfForm = psfForm;
		init();
		loadForm();
		showForm();
	}

	void init()
	{
		this.gblLayout = new GridBagLayout();
		this.gbcConst = new GridBagConstraints();
		this.gbcConst.insets = new Insets(5, 5, 5, 5);

		this.jlAddMac = new JLabel("MAC: ");
		this.jlRmvOwner = new JLabel("Owner: ");

		this.jtfAddMac = new JTextField[MAC_ADDR_LEN];
		for(int i=0; i<this.jtfAddMac.length; i++) this.jtfAddMac[i] = new JTextField(2);
		this.jtfRmvOwner = new JTextField(10);

		this.jpAdd = new JPanel();
		this.jpAdd.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED), "Add MAC with Priority"));
		  //Border with title and raised line
		
		this.jpRmv = new JPanel();
		this.jpRmv.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED), "Remove an Owner with Priority"));
		  //Border with title and raised line
		
		this.jpConfigPanel = new JPanel();
		this.jpConfigPanel.setLayout(this.gblLayout);
		
		this.jbCancel = new JButton("Cancel");
		this.jbOk = new JButton("Ok");
		this.jbApply = new JButton("Apply");

		this.jbCancel.addActionListener(this);
		this.jbOk.addActionListener(this);
		this.jbApply.addActionListener(this);
	}

	void loadForm()
	{
		this.jpAdd.setLayout(new GridLayout(1, 2));
		this.jpAdd.add(this.jlAddMac);
		JPanel jpAddText = new JPanel();
		for(int i=0; i<this.jtfAddMac.length; i++) jpAddText.add(this.jtfAddMac[i]);
		this.jpAdd.add(jpAddText);

		this.gbcConst.weightx = 1.0;
		this.gbcConst.weighty = 1.0;
		this.gbcConst.anchor = GridBagConstraints.NORTH;
		this.gbcConst.gridwidth = GridBagConstraints.REMAINDER;
		this.gbcConst.fill = GridBagConstraints.BOTH;
		this.gblLayout.setConstraints(this.jpAdd, this.gbcConst);
		this.jpConfigPanel.add(this.jpAdd);
		
		this.jpRmv.setLayout(new GridLayout(1, 2));
		this.jpRmv.add(this.jlRmvOwner);
		this.jpRmv.add(this.jtfRmvOwner);

		this.gbcConst.weightx = 1.0;
		this.gbcConst.weighty = 1.0;
		this.gbcConst.anchor = GridBagConstraints.CENTER;
		this.gbcConst.gridwidth = GridBagConstraints.REMAINDER;
		this.gbcConst.fill = GridBagConstraints.BOTH;
		this.gblLayout.setConstraints(this.jpRmv, this.gbcConst);
		this.jpConfigPanel.add(this.jpRmv);

		JPanel jpButton = new JPanel(new GridLayout(1, 4, 5, 5));
		jpButton.add(new JPanel());
		jpButton.add(this.jbOk);
		jpButton.add(this.jbCancel);
		jpButton.add(this.jbApply);

		this.gbcConst.weightx = 0.0;
		this.gbcConst.fill = GridBagConstraints.NONE;
		this.gbcConst.anchor = GridBagConstraints.SOUTHEAST;
		this.gblLayout.setConstraints(jpButton, this.gbcConst);
		this.jpConfigPanel.add(jpButton);
		
		this.getContentPane().add(this.jpConfigPanel);
	}
	
	void showForm()
	{
		this.pack();
		Rectangle bounds = super.getOwner().getGraphicsConfiguration().getBounds();
		this.setLocation(bounds.width/2 - this.getWidth()/2, bounds.height/2 - this.getHeight()/2);
		this.setVisible(true);

	}	


	public void actionPerformed(ActionEvent e) 
	{
		Object obSource = e.getSource();
		if(obSource == this.jbCancel)
			this.dispose();
		else if(obSource == this.jbOk) {
			this.applySettings();
			this.dispose();
		}
		else if(obSource == this.jbApply)
			this.applySettings();
	}

	private void applySettings()
	{
		boolean tbHasModified = false;

		String strText = this.jtfAddMac[0].getText();		
		String strMac  = this.checkAndModifyString(strText);
		String strNull = "00";
		
		for(int i=1; i<this.jtfAddMac.length; i++) {
			strText  = this.jtfAddMac[i].getText();
			strMac  += (":"+this.checkAndModifyString(strText));
			strNull += ":00";
		}
		if(!strMac.equals(strNull)) {

			boolean thereIs = this.pdpValues.searchPriorityMAC(strMac);
			
			if(!thereIs) {
				
				this.pdpValues.addPriorityMAC(strMac);
				
				tbHasModified = true;
			}
		}

		for(int i=0; i<this.jtfAddMac.length; i++) this.jtfAddMac[i].setText("");

		String strOwner = jtfRmvOwner.getText();
		if(!strOwner.equals("")) {
			
			int indexOwner = Integer.parseInt(strOwner);
			
			if(!tbHasModified)
				tbHasModified = this.pdpValues.removePriorityMAC(indexOwner);
			
			this.jtfRmvOwner.setText("");
		}

		if(tbHasModified) {
			this.psStatus.writeDriverInfo();
			this.psfForm.setPriorityEntries(this.pdpValues.getAllPriorityMAC());
		}
	}
	
	private String checkAndModifyString(String strText) {
		if(strText.length()==0) strText = "00";
		else if(strText.length()==1) strText += "0";
		else strText = strText.substring(0,2);
		
		String strRet = "";
		for(int i=0; i<2; i++) {
			char ch = strText.charAt(i);
			
			if(Character.isLetter(ch)) {
				ch = Character.toLowerCase(ch);
				
				if(ch<'a' || ch>'f') ch = '0';
			}
			else if(!Character.isDigit(ch)) ch = '0';
			
			strRet += ch;
		}
		
		return strRet;
	}
}
